package com.flyfish.framework.auditor;

import com.flyfish.framework.context.UserContext;
import com.flyfish.framework.domain.base.AuditDomain;
import com.flyfish.framework.domain.base.IUser;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;

@Component
@RequiredArgsConstructor
public class ReactiveOperationAuditor implements ReactiveBeanAuditor<AuditDomain> {

    private final UserContext userContext;

    /**
     * 对实体进行审查，并补全相关字段
     *
     * @param data 原数据
     * @return 结果
     */
    @Override
    public Mono<AuditDomain> audit(AuditDomain data) {
        return ReactiveSecurityContextHolder.getContext()
                .map(ctx -> ((IUser) ctx.getAuthentication().getPrincipal()).toUser())
                .map(user -> {
                    if (StringUtils.isNotBlank(data.getId())) {
                        data.setModifierId(user.getId());
                        data.setModifier(user.getName());
                    } else {
                        data.setCreatorId(user.getId());
                        data.setCreator(user.getName());
                        data.setModifierId(user.getId());
                    }
                    return data;
                })
                .defaultIfEmpty(data);
    }
}
