package com.flyfish.framework.domain.authorized;

import com.flyfish.framework.builder.CriteriaBuilder;
import com.flyfish.framework.domain.base.NameLikeQo;
import com.flyfish.framework.domain.po.Department;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.data.mongodb.core.query.Criteria;

import java.util.Collections;
import java.util.Set;

/**
 * 公共方法封装
 *
 * @param <T> 泛型
 */
@Setter
public abstract class AbstractAuthorizedQo<T extends AuthorizedDomain> extends NameLikeQo<T> {

    // 是否已发布
    protected Boolean published;

    /**
     * 获取可见的权限ids
     *
     * @return 结果
     */
    protected Set<String> authorizeIds() {
        if (user instanceof AuthorizedUserDetails) {
            return ((AuthorizedUserDetails) user).getAuthorityCodes();
        }
        return Collections.singleton(Department.PUBLIC);
    }

    /**
     * 构造发布状态查询
     *
     * @return 结果
     */
    protected Criteria withPublished() {
        if (null != published) {
            if (BooleanUtils.isTrue(published)) {
                return Criteria.where("published").is(true);
            }
            return Criteria.where("$or").is(
                    CriteriaBuilder.createCriteriaList(
                            Criteria.where("published").isNull(),
                            Criteria.where("published").is(false)
                    )
            );
        }
        return null;
    }

}
