package com.flyfish.framework.domain.authorized;

import com.flyfish.framework.builder.CriteriaBuilder;
import com.flyfish.framework.enums.UserType;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.data.mongodb.core.query.Criteria;

/**
 * 带鉴权的查询实体，主要以部门隔绝
 *
 * @param <T>
 */
@Getter
@Setter
public class AuthorizedQo<T extends AuthorizedDomain> extends AbstractAuthorizedQo<T> {

    @Override
    public CriteriaBuilder<T> criteriaBuilder() {
        // 超级管理员拥有查看所有草稿的权限
        if (user.getType() == UserType.SUPER_ADMIN) {
            return super.criteriaBuilder().with(this::withPublished);
        }
        // 查询草稿，只查询自己的
        if (BooleanUtils.isFalse(published)) {
            return super.criteriaBuilder()
                    .with(this::withPublished)
                    .with(() -> Criteria.where("creatorId").is(user.getId()));
        }
        // 普通查询，根据权限配置查询
        return super.criteriaBuilder()
                .with(this::withPublished)
                .with(() -> Criteria.where("$or").is(
                        CriteriaBuilder.createCriteriaList(
                                Criteria.where("authorizeId").in(authorizeIds()),
                                Criteria.where("creatorId").is(user.getId())
                                        .and("authorizeId").in(((AuthorizedUserDetails) user).getVisibleDeparts())
                        )
                ));
    }

}
