package com.flyfish.framework.domain.authorized.advanced;

import com.flyfish.framework.builder.CriteriaBuilder;
import com.flyfish.framework.domain.authorized.AbstractAuthorizedQo;
import com.flyfish.framework.domain.authorized.AuthorizedUserDetails;
import com.flyfish.framework.enums.UserType;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.BooleanUtils;
import org.bson.types.ObjectId;
import org.springframework.data.mongodb.core.query.Criteria;

/**
 * 拥有归属权的授权查询
 *
 * @param <T> 泛型
 * @author wangyu
 * 将会额外查询归属者的内容
 */
@Getter
@Setter
public class OwnedAuthorizedQo<T extends OwnedAuthorizedDomain> extends AbstractAuthorizedQo<T> {

    @Override
    public CriteriaBuilder<T> criteriaBuilder() {
        // 超级管理员拥有查看所有草稿的权限
        if (user.getType() == UserType.SUPER_ADMIN) {
            return super.criteriaBuilder().with(this::withPublished);
        }
        // 查询草稿，只查询自己的
        if (BooleanUtils.isFalse(published)) {
            return super.criteriaBuilder()
                    .with(this::withPublished)
                    .with(() -> Criteria.where("$or").is(
                            CriteriaBuilder.createCriteriaList(
                                    Criteria.where("creatorId").is(user.getId()),
                                    Criteria.where("owners.$id").is(new ObjectId(user.getId()))
                            )
                    ));
        }
        // 普通查询，根据权限配置查询
        return super.criteriaBuilder()
                .with(this::withPublished)
                .with(() -> Criteria.where("$or").is(
                        CriteriaBuilder.createCriteriaList(
                                Criteria.where("authorizeId").in(authorizeIds()),
                                Criteria.where("creatorId").is(user.getId())
                                        .and("authorizeId").in(((AuthorizedUserDetails) user).getVisibleDeparts()),
                                Criteria.where("owners.$id").is(new ObjectId(user.getId()))
                        )
                ));
    }
}
