package com.flyfish.framework.beans.meta.parser;

import com.flyfish.framework.beans.meta.parser.chain.BeanPropertyAnnotationBatchChain;
import com.flyfish.framework.beans.meta.parser.chain.BeanPropertyAnnotationChain;
import org.springframework.core.annotation.MergedAnnotation;
import org.springframework.core.annotation.MergedAnnotations;

import java.lang.annotation.Annotation;

/**
 * 基于spring annotations最高级封装
 * 1. 全optional机制
 * 2. 完整链式调用
 * 3. 支持或、与操作
 *
 * @author wangyu
 */
public interface BeanPropertyAnnotations {

    /**
     * 判断是否有某个注解
     *
     * @param annotationType 注解类型
     * @param <A>            泛型
     * @return 结果
     */
    <A extends Annotation> boolean isPresent(Class<A> annotationType);

    /**
     * 获取纯粹的spring注解属性表
     *
     * @return 结果
     */
    MergedAnnotations annotations();

    /**
     * 判断某个注解是否存在，存在则交给consumer进行处理，仅取得找到的第一个注解
     *
     * @param annotationType 注解类型
     * @param <A>            泛型
     * @return 结果
     */
    <A extends Annotation> BeanPropertyAnnotationChain<A, MergedAnnotation<A>> as(Class<A> annotationType);

    /**
     * 判断某个注解是否存在，存在则交给consumer进行处理，取得所有注解并自动判断复数形式
     *
     * @param annotationType 注解类型
     * @param listType       列表类型
     * @param <A>            泛型
     * @param <L>            列表泛型
     * @return 结果
     */
    <A extends Annotation, L extends Annotation> BeanPropertyAnnotationBatchChain<A, MergedAnnotation<A>> as(
            Class<A> annotationType, Class<L> listType);

    /**
     * 设置上一步结果
     *
     * @param success 是否成功
     * @return 结果
     */
    BeanPropertyAnnotations last(boolean success);

    /**
     * 返回上一步成功结果
     *
     * @return 结果
     */
    boolean last();

    /**
     * 返回空的运行链
     *
     * @param <A> 注解泛型
     * @return 结果
     */
    <A extends Annotation, E> BeanPropertyAnnotationChain<A, E> empty();

    /**
     * 返回空的运行链
     *
     * @param <A> 注解泛型
     * @return 结果
     */
    <A extends Annotation, E> BeanPropertyAnnotationBatchChain<A, E> batchEmpty();
}
