package com.flyfish.framework.beans.meta.parser.chain.impl;

import com.flyfish.framework.beans.meta.parser.BeanPropertyAnnotations;
import com.flyfish.framework.beans.meta.parser.chain.BeanPropertyAnnotationBatchChain;
import org.springframework.data.util.CastUtils;

import java.lang.annotation.Annotation;
import java.util.Collections;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;

/**
 * 单例的，空实现
 *
 * @param <A> 注解泛型
 * @param <T> 实际泛型
 */
public class EmptyAnnotationBatchChain<A extends Annotation, T> extends BasicAnnotationChainSupport implements BeanPropertyAnnotationBatchChain<A, T> {

    public EmptyAnnotationBatchChain(BeanPropertyAnnotations parent) {
        super(parent);
    }

    /**
     * 映射转换，将转换每一条
     *
     * @param mapper 映射器
     * @return 结果
     */
    @Override
    public <R> BeanPropertyAnnotationBatchChain<A, R> map(Function<T, R> mapper) {
        return CastUtils.cast(this);
    }

    /**
     * 过滤，不满足的将过滤，如果为空，将empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationBatchChain<A, T> filter(Predicate<T> filter) {
        return this;
    }

    /**
     * 过滤，不满足的将过滤，如果为空，将empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationBatchChain<A, T> filter(Supplier<Boolean> filter) {
        return this;
    }

    /**
     * 如果存在，则做
     *
     * @param consumer 消费者
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationBatchChain<A, T> then(Consumer<List<T>> consumer) {
        return this;
    }

    /**
     * 存在时执行逻辑，不需要结果
     *
     * @param handler 处理器
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationBatchChain<A, T> exists(Runnable handler) {
        return this;
    }

    /**
     * 不存在做的事情
     *
     * @param fallback 补偿逻辑
     * @return 结果
     */
    @Override
    public BeanPropertyAnnotationBatchChain<A, T> empty(Runnable fallback) {
        if (!parent.last()) {
            fallback.run();
        }
        return this;
    }

    /**
     * 获取所有结果为list
     *
     * @return 结果
     */
    @Override
    public List<A> synthesize() {
        parent.last(false);
        return Collections.emptyList();
    }
}
